/************************************************************************
**
**  Copyright (C) 2015-2025 Kevin B. Hendricks, Stratford, Ontario Canada
**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
**
**  This file is part of Sigil.
**
**  Sigil is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  Sigil is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with Sigil.  If not, see <http://www.gnu.org/licenses/>.
**
*************************************************************************/

#pragma once
#ifndef SVGTAB_H
#define SVGTAB_H

#include "MiscEditors/ClipEditorModel.h"
#include "Tabs/TextTab.h"
#include "ViewEditors/CodeViewEditor.h"

class SVGResource;

class SVGTab : public TextTab
{
    Q_OBJECT

public:
    SVGTab(SVGResource *resource, int line_to_scroll_to = -1, int position_to_scroll_to = -1, QWidget *parent = 0);

public slots:
    void HandleViewImage(const QUrl &url);

signals:

    /**
     * Emitted when CV wants to View Images
     * 
     * @param url The URL of the Image using our book:/// scheme.
     */

    void ViewImageRequest(const QUrl &url);
    void TabUpdated();

private slots:
    void EmitTabUpdated();
};

#endif // SVGTAB_H
